package com.jiuqi.mde;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.jiuqi.mde.MDEDataInfo;
import com.jiuqi.mde.MDEDimValueSet;
import com.jiuqi.mde.MDEDimension;
import com.jiuqi.mde.MDEDimensionSet;
import com.jiuqi.mde.MDEException;
import com.jiuqi.mde.MDEField;
import com.jiuqi.mde.MDEFieldSet;
import com.jiuqi.mde.MDEIndex;
import com.jiuqi.util.Guid;
import java.util.Arrays;

public class MDERecord {
  public Object refObj;
  
  protected MDEDataInfo dataInfo;
  
  protected MDEFieldSet fields;
  
  protected MDEDimValueSet dimValues;
  
  protected Object[] fieldValues;
  
  protected int recordFlag;
  
  public static final int RF_UNKNOWN = 0;
  
  public static final int RF_LOADED = 1;
  
  public static final int RF_MODIFIED = 2;
  
  public static final int RF_INSERTING = 4;
  
  public static final int RF_DELETING = 8;
  
  public static final int RF_DELETED = 16;
  
  public MDERecord(MDEDataInfo dataInfo) {
    this.dataInfo = dataInfo;
    this.fields = dataInfo.fields();
    this.fieldValues = new Object[dataInfo.fields().size()];
  }
  
  public final MDEDimensionSet dimensions() {
    return this.dataInfo.dimensions();
  }
  
  public final MDEFieldSet fields() {
    return this.dataInfo.fields();
  }
  
  public final MDEDimValueSet getDimValues() {
    if (this.dimValues == null)
      refreshDimValues(null); 
    return this.dimValues;
  }
  
  public void setDimValues(MDEDimValueSet dimValues, MDEIndex dvIndex) {
    if (dvIndex != null && this.dimValues != null)
      dvIndex.remove(this.dimValues); 
    getDimValues().assignValues(dimValues);
    for (int i = 0; i < dimensions().size(); i++) {
      MDEDimension dimension = dimensions().get(i);
      MDEField field = this.fields.findByFieldName(dimension.fieldName);
      if (field != null)
        setFieldValue(field, dimValues.getValue(dimension)); 
    } 
    if (dvIndex != null)
      dvIndex.addlast(this.dimValues, this); 
  }
  
  public void refreshDimValues(MDEIndex dvIndex) {
    if (this.dimValues != null && !isModified())
      return; 
    if (dvIndex != null && this.dimValues != null)
      dvIndex.remove(this.dimValues); 
    if (this.dimValues == null)
      this.dimValues = new MDEDimValueSet(this.dataInfo.dimensions()); 
    for (int i = 0; i < dimensions().size(); i++) {
      MDEDimension dimension = dimensions().get(i);
      MDEField field = this.fields.findByFieldName(dimension.fieldName);
      if (field != null)
        this.dimValues.setValue(dimension, getFieldValue(field)); 
    } 
    if (dvIndex != null)
      dvIndex.addlast(this.dimValues, this); 
  }
  
  protected final Object getFieldValue(int index) {
    return this.fieldValues[index];
  }
  
  protected final Object getFieldValue(MDEField field) {
    try {
      int index = this.fields.indexOf(field);
      return this.fieldValues[index];
    } catch (ArrayIndexOutOfBoundsException arrayIndexOutOfBoundsException) {
      throw new ArrayIndexOutOfBoundsException("找不到字段：" + field.fieldName);
    } 
  }
  
  protected final void setFieldValue(int index, Object value) {
    if ((value == null && this.fieldValues[index] != null) || (
      value != null && !value.equals(this.fieldValues[index])))
      this.recordFlag |= 0x2; 
    this.fieldValues[index] = value;
  }
  
  protected final void setFieldValue(MDEField field, Object value) {
    int index = this.fields.indexOf(field);
    if (index < 0)
      throw new ArrayIndexOutOfBoundsException("找不到字段：" + field.fieldName); 
    if ((value == null && this.fieldValues[index] != null) || (
      value != null && !value.equals(this.fieldValues[index])))
      this.recordFlag |= 0x2; 
    this.fieldValues[index] = value;
  }
  
  public final void copyFieldValue(com.jiuqi.mde.MDERecord source, int index) {
    Object value = source.fieldValues[index];
    if ((value == null && this.fieldValues[index] != null) || (
      value != null && !value.equals(this.fieldValues[index])))
      this.recordFlag |= 0x2; 
    this.fieldValues[index] = value;
  }
  
  public final void generateIdenty() {
    setFieldValue(this.dataInfo.getIndentyField(), Guid.newGuid());
  }
  
  public boolean hasData() {
    for (int i = 0; i < this.fieldValues.length; i++) {
      MDEField field = this.fields.get(i);
      if (field.object != null && this.fieldValues[i] != null)
        return true; 
    } 
    return false;
  }
  
  public final boolean isInserted() {
    return ((this.recordFlag & 0x4) != 0);
  }
  
  public final boolean isModified() {
    return ((this.recordFlag & 0x2) != 0);
  }
  
  public final boolean isDeleting() {
    return ((this.recordFlag & 0x8) != 0);
  }
  
  public final boolean isDeleted() {
    return ((this.recordFlag & 0x10) != 0);
  }
  
  public final void setIsInserted() {
    this.recordFlag = 6;
  }
  
  public final void clearRecordFlag() {
    this.recordFlag = 0;
  }
  
  public final String getRecordState() {
    if (isDeleted())
      return "Deleted"; 
    if (isDeleting())
      return "Deleting"; 
    if (isInserted())
      return "Inserted"; 
    if (isModified())
      return "Modified"; 
    return "Default";
  }
  
  protected void beforeNewRecord() throws MDEException {}
  
  protected void afterNewRecord() throws MDEException {
    this.recordFlag = 4;
  }
  
  protected void beforeLoadRecord() throws MDEException {}
  
  protected void afterLoadRecord() throws MDEException {
    this.recordFlag = 1;
  }
  
  public void checkModification() throws MDEException {}
  
  protected void beforeSaveRecord() throws MDEException {}
  
  protected void afterSaveRecord() throws MDEException {
    this.recordFlag = 1;
  }
  
  protected void beforeDeleteRecord() throws MDEException {}
  
  protected void afterDeleteRecord() throws MDEException {
    this.recordFlag = 16;
  }
  
  protected void calculateDefaultValues() throws MDEException {}
  
  public void delete() {
    this.recordFlag |= 0x8;
  }
  
  public final void emptyRecord() {
    this.recordFlag = 0;
    Arrays.fill(this.fieldValues, (Object)null);
  }
  
  public final void assignValues(com.jiuqi.mde.MDERecord source) {
    for (int i = 0; i < this.fieldValues.length; i++)
      this.fieldValues[i] = source.fieldValues[i]; 
    this.recordFlag |= 0x2;
  }
  
  public final void negateValues() {
    for (int i = 0; i < this.fieldValues.length; i++) {
      Object value = this.fieldValues[i];
      if (value instanceof Integer) {
        this.fieldValues[i] = new Integer(
            -((Integer)this.fieldValues[i]).intValue());
      } else if (value instanceof Double) {
        this.fieldValues[i] = new Double(
            -((Double)this.fieldValues[i]).doubleValue());
      } 
    } 
    this.recordFlag |= 0x2;
  }
}
